from rest_framework.permissions import BasePermission
from .models import Token
from .HMACToken import HmacToken, TokenPeriod
from django.contrib.auth import get_user_model
from django.conf import settings

HMAC_LOGIN_HEADER = getattr(settings, 'HMAC_LOGIN_HEADER', 'HMAC-Login')
HMAC_TOKEN_HEADER = getattr(settings, 'HMAC_TOKEN_HEADER', 'HMAC-Token')
HMAC_TIMES_HEADER = getattr(settings, 'HMAC_TOKEN_HEADER', 'HMAC-Times') 
HMAC_PERIOD = getattr(settings, 'HMAC_PERIOD', TokenPeriod.day)
HMAC_HASH_FUNC = getattr(settings, 'HMAC_HASH_FUNC', 'md5')

User = get_user_model()

class TokenPermission(BasePermission):
    message = 'Wrong or expired temporary token'

    def has_permission(self, request, view):
        if 'HMAC-Login' not in request.headers or 'HMAC-Times' not in request.headers or 'HMAC-Token' not in request.headers:
            return False
        token = request.headers[HMAC_TOKEN_HEADER]
        login = request.headers[HMAC_LOGIN_HEADER]
        times = request.headers[HMAC_TIMES_HEADER]
        users = User.objects.filter(username=login).all()
        if len(users) == 0:
            return False
        tokens = Token.objects.filter(user=users[0]).all() 
        if len(tokens) == 0:
            return False
        crypt = HmacToken(tokens[0].token, HMAC_PERIOD, HMAC_HASH_FUNC)
        return crypt.checkToken(login, times, token)

    def has_object_permission(self, request, view, user):
        if 'HMAC-Login' not in request.headers or 'HMAC-Times' not in request.headers or 'HMAC-Token' not in request.headers:
            return False
        token = request.headers[HMAC_TOKEN_HEADER]
        login = request.headers[HMAC_LOGIN_HEADER]
        times = request.headers[HMAC_TIMES_HEADER]
        tokens = Token.objects.filter(user=user).all() 
        if len(tokens) == 0:
            return False
        crypt = HmacToken(tokens[0].token, HMAC_PERIOD, HMAC_HASH_FUNC)
        return crypt.checkToken(login, times, token)
        
        




import hashlib
import enum
import calendar
from datetime import datetime
from django.conf import settings

today = datetime.now().date()

class TokenPeriod(enum.IntEnum):
    minute = 60
    hour = minute * 60
    day = hour * 24
    week = 7 * day
    month = calendar.monthrange(today.year, today.month)[1] * day
    year = 366 * day if calendar.isleap(today.year) else 365 * day

class HmacToken:
    def __init__(self, salt, period, hash_func):
        self.salt = salt
        self.period = period
        self.hash_func = hash_func

    def getToken(self, login):
        times = str(int(datetime.today().timestamp()))
        h = hashlib.new(self.hash_func)
        h.update((login + times + self.salt).encode())
        return (times, h.hexdigest())

    def checkToken(self, login, times, token):
        h = hashlib.new(self.hash_func)
        h.update((login + times + self.salt).encode())
        predict = h.hexdigest()
        return token == predict and datetime.today().timestamp() - int(times) <= self.period


from rest_framework import serializers
from .HMACToken import HmacToken, TokenPeriod
from .models import Token
from django.conf import settings

class TokenSerializer(serializers.BaseSerializer):
    login = serializers.CharField(max_length=255)
    times = serializers.CharField(max_length=255)
    token = serializers.CharField(max_length=256)

    def __init__(self, *args, **kwargs):
        kwargs['data'] = {}
        if 'user' in kwargs:
           kwargs['data']['user'] = kwargs['user']
        kwargs.pop('user', None)
        super().__init__(**kwargs)

    def to_representation(self, data):
        output = {}
        output['login'] = self.login
        output['times'] = self.times
        output['token'] = self.token
        return output

    def to_internal_value(self, data):
        if 'user' not in data:
            raise serializers.ValidationError({"error" : "User is missing"})
        HMAC_PERIOD = getattr(settings, 'HMAC_PERIOD', TokenPeriod.day)
        HMAC_HASH_FUNC = getattr(settings, 'HMAC_HASH_FUNC', 'md5')
        output = {}
        user = data['user']
        tokens = Token.objects.filter(user=user).all()
        token = '' 
        if len(tokens) == 0:
            obj = Token.objects.create(user=user)
            obj.save()
            token = obj.token
        else:
            token = tokens[0].token
        login = user.username
        crypt = HmacToken(token, HMAC_PERIOD, HMAC_HASH_FUNC)
        times, temp_token = crypt.getToken(login)
        self.login = login
        self.times = times
        self.token = temp_token
        return self


from django.db import models
from django.conf import settings
import random

class Token(models.Model):
    token = models.CharField(max_length=32)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)

    def save(self, *args, **kwargs):
        str1 = '123456789'
        str2 = 'qwertyuiopasdfghjklzxcvbnm/*:#@!?&$'
        str3 = str2.upper()
        str4 = str1 + str2 + str3
        ls = list(str4)
        random.shuffle(ls)
        token = ''.join([random.choice(ls) for x in range(32)])
        self.token = token
        super().save(*args, **kwargs) 